@@ -33,8 +33,16 @@ class ServerContext:
         self.attrs = {}
 
     async def setup(self):
+        await self.redis.connect()
         await self.event_handler_manager.setup()
 
+    async def setup_route(self, route):
+        await self.redis.connect()
+        await self.event_handler_manager.setup()
+
+    async def run(self):
+        await self.event_handler_manager.run()
+
     def module_path(self, path):
         return path.resolve() if path.is_absolute() else Path(__file__).parent.joinpath(path).resolve()
 
