@@ -1,4 +1,6 @@
-from ducts.event import EventHandler
+from ducts.spi import EventHandler
+
+import time
 
 import asyncio
 
@@ -15,14 +17,14 @@ class Handler(EventHandler):
         self.conf = configure_module(config)
 
     def setup(self, handler_spec, manager):
-        handler_spec.set_description('死活監視用メインループを開始します。最初に一度だけ自動的に呼び出されます。')
+        handler_spec.set_description('死活監視用メインループを開始します。最初に一度だけ自動的に呼び出されます。サーバ時刻を定期的に返します。')
         return handler_spec
 
     async def handle(self, event):
-        yield True
+        yield (time.time(), event.session.timestamp)
         while not await event.session.is_closed():
             await asyncio.sleep(self.conf.alive_monitoring_interval)
-            yield True
+            yield (time.time(), event.session.timestamp)
         
     
 
