@@ -3,10 +3,14 @@
 import inspect
 import functools
 import itertools
+import hashlib
+from collections import defaultdict
 
 import asyncio
 import aioredis
 
+from .spi import RedisClient as RedisClientAPI
+
 from ifconf import configure_module, config_callback, configure_main
 
 import logging
@@ -18,7 +22,7 @@ def config(loader):
     #loader.add_attr('redis_uri_logging', 'redis://localhost:6379/1?encoding=utf-8', help='redis uri to connect') # or uri 'unix:///path/to/redis/socket?db=1'
 
 
-class RedisClient:
+class RedisClient(RedisClientAPI):
 
     def __init__(self, loop):
         self.conf = configure_module(config)
@@ -28,6 +32,7 @@ class RedisClient:
         self.count = 0
         self._subs = {}
         self._psubs = {}
+        self._script_sha1 = sha1_loader()
 
     async def connect(self):
         self.conn = await aioredis.create_redis_pool(self.conf.redis_uri_main, minsize=1, maxsize=1)
@@ -66,13 +71,11 @@ class RedisClient:
                 break
         return ret
 
-    #need to await
-    def execute(self, cmd, *args):
-        return self.conn.execute(cmd, *args, encoding=None)
+    async def execute(self, cmd, *args):
+        return await self.conn.execute(cmd, *args, encoding=None)
             
-    #need to await
-    def execute_str(self, cmd, *args):
-        return self.conn.execute(cmd, *args, encoding='UTF-8')
+    async def execute_str(self, cmd, *args):
+        return await self.conn.execute(cmd, *args, encoding='UTF-8')
             
     async def subscribe(self, key):
         if key in self._subs:
@@ -121,20 +124,30 @@ class RedisClient:
         ret = await self.execute_str('XREVRANGE', streamkey, stream_id, stream_id, 'COUNT', 1)
         return {v[0] : v[1] for v in zip(*[iter(ret[0][1])]*2)} if ret else {}
 
+    async def xget_str_with_id(self, streamkey, stream_id):
+        ret = await self.execute_str('XREVRANGE', streamkey, stream_id, stream_id, 'COUNT', 1)
+        return (ret[0][0], dict({v[0] : v[1] for v in zip(*[iter(ret[0][1])]*2)})) if ret else ('', {})
+
     async def xlast_str(self, streamkey):
         ret = await self.execute_str('XREVRANGE', streamkey, '+', '-', 'COUNT', 1)
         return {v[0] : v[1] for v in zip(*[iter(ret[0][1])]*2)} if ret else {}
 
+    async def xlast_str_with_id(self, streamkey):
+        ret = await self.execute_str('XREVRANGE', streamkey, '+', '-', 'COUNT', 1)
+        return (ret[0][0], dict({v[0] : v[1] for v in zip(*[iter(ret[0][1])]*2)})) if ret else ('', {})
+
+    async def hget_all_str(self, key):
+        ret = await self.execute_str('HGETALL', key)
+        return {v[0] : v[1] for v in zip(*[iter(ret)]*2)} if ret else {}
 
     #https://github.com/aio-libs/aioredis/issues/310
     #https://github.com/aio-libs/aioredis/issues/369
     #https://github.com/aio-libs/aioredis/pull/373
     async def psub_and_xrange_str(self, subkey, streamkey, last_count = 0):
-        logger.info("pubsub_channels=%s", self.conn_for_subscription.pubsub_channels())
-        
+        #logger.info("pubsub_channels=%s", self.conn_for_subscription.pubsub_channels())
         #ch = (await self.psubscribe(subkey))[0]
         ch = (await self.psubscribe(subkey))
-        logger.info('PSUBSCRIBE|CHANNEL=%s', ch)
+        logger.debug('PSUBSCRIBE|CHANNEL=%s', ch)
         last_id = '0'
         if last_count > 0:
             ret = await self.execute_str('XREVRANGE', streamkey, '+', '-', 'COUNT', last_count)
@@ -142,9 +155,12 @@ class RedisClient:
                 last_id = result[0]
                 yield {v[0] : v[1] for v in zip(*[iter(result[1])]*2)}
         async for msg in ch.iter():
-            logger.info('PSUBSCRIBE|CHANNEL=%s|MSG=%s:%s', ch, msg[0], type(msg[1]))
+            if msg is None:
+                logger.debug('PSUBSCRIBE|CHANNEL=%s|CLOSED', ch)
+                break
+            logger.debug('PSUBSCRIBE|CHANNEL=%s|MSG=%s:%s', ch, msg[0], type(msg[1]))
             ret = await self.execute_str('XRANGE', streamkey, last_id+'1', '+')
-            logger.info('XRANGE|STREAM=%s|LAST_ID=%s|ret=%s', streamkey, last_id, ret)
+            logger.debug('XRANGE|STREAM=%s|LAST_ID=%s|ret=%s', streamkey, last_id, ret)
             if ret is None:
                 continue
             for result in ret:
@@ -156,13 +172,42 @@ class RedisClient:
         ch = (await self.psubscribe(subkey))
         logger.debug('PSUBSCRIBE|CHANNEL=%s', ch)
         async for msg in ch.iter():
-            logger.debug('PSUBSCRIBE|CHANNEL=%s|MSG=%s:%s', ch, msg[0], msg[1])
-            ret = await self.execute_str('XRANGE', streamkey, msg[1], msg[1])
-            logger.debug('XRANGE|STREAM=%s|ID=%s|RET=%s', streamkey, msg, ret)
-            if ret is None:
-                continue
-            for result in ret:
-                yield {v[0] : v[1] for v in zip(*[iter(result[1])]*2)}
+            if msg is None:
+                logger.debug('PSUBSCRIBE|CHANNEL=%s|MSG=None', ch)
+                break
+            else:
+                logger.debug('PSUBSCRIBE|CHANNEL=%s|MSG=%s:%s', ch, msg[0], msg[1])
+                ret = await self.execute_str('XRANGE', streamkey, msg[1], msg[1])
+                logger.debug('XRANGE|STREAM=%s|ID=%s|RET=%s', streamkey, msg, ret)
+                if ret is None:
+                    continue
+                for result in ret:
+                    yield {v[0] : v[1] for v in zip(*[iter(result[1])]*2)}
+
+    async def scan_for(self, pattern, count = 10):
+        cursor = 0
+        while True:
+            ret = await self.execute_str('SCAN', cursor, 'MATCH', pattern, 'COUNT', count)
+            for key in ret[1]:
+                yield key
+            cursor = ret[0]
+            if cursor == '0':
+                break
+
+    async def xrevrange_str(self, streamkey, start = '+', stop = '-', count = 10):
+        ret = await self.execute_str('XREVRANGE', streamkey, start, stop, 'COUNT', count)
+        if ret is None:
+            return ()
+        return ((row[0], {pair[0] : pair[1] for pair in zip(*[iter(row[1])]*2)}) for row in ret)
+
+    async def evalsha(self, script, numkeys, *params):
+        #print(script)
+        sha1 = self._script_sha1(script)
+        try:
+            return await self.execute('EVALSHA', sha1, numkeys, *params)
+        except Exception as e:
+            logger.debug('EVALSHA|FAILED_LOAD_CACHE|%s', e)
+            return await self.execute('EVAL', script, numkeys, *params)
             
     
     async def close(self):
@@ -187,6 +232,19 @@ class RedisClient:
             ret = await conn.publish('test', key)
     '''
 
+class sha1_loader:
+    def __init__(self):
+        self.db = {}
+            
+    def __call__(self, script):
+        try:
+            return self.db[script]
+        except KeyError:
+            sha1 = hashlib.sha1(script.encode('UTF-8')).hexdigest()
+            self.db[script] = sha1
+            return sha1
+
+    
 #https://github.com/aio-libs/aioredis/issues/310
 #https://stackoverflow.com/questions/54159292/how-to-implement-single-producer-multi-consumer-with-aioredis-pub-sub
 class ChannelForMultiConsumer:
