@@ -43,6 +43,8 @@ def config(loader):
     loader.add_attr_int('httpd_port', 8080, help='inet port to bind')
     loader.add_attr_path('local_path_static', Path('./htdocs'), help='local path for static root')
     loader.add_attr_path('local_path_libs', Path('./libs'), hidden=True, help='local path for html/javascript files')
+    loader.add_attr('wsd_host', '', help='When this value is set, ws_host and api_host is overritten by this value.')
+    loader.add_attr_boolean('wsd_secure', False, help='When this value is True, wss and https is used for wsd')
 
     loader.add_attr_path('module_path_libs', Path('./libs'), hidden=True, help='module path for html/javascript files')
     loader.add_attr_list('web_lib_file_extensions', ['.js', '.html', '.css'], hidden=True, help='web library file extensions')
@@ -58,8 +60,8 @@ def config(loader):
     loader.add_attr('relative_path_event_spec', 'events.json', hidden=True, help='where event descriptions json is located')
     loader.add_attr('relative_path_api', 'api/v1/', hidden=True, help='')
     loader.add_attr('relative_path_websocket', 'ws/v1', hidden=True, help='')
-    loader.add_attr('api_path_event', 'event', hidden=True, help='')
-    loader.add_attr('api_path_event_id', 'event/{event_id}', hidden=True, help='')
+    #loader.add_attr('api_path_event', 'event', hidden=True, help='')
+    #loader.add_attr('api_path_event_id', 'event/{event_id}', hidden=True, help='')
     loader.add_attr('jsonp_callback_query', 'callback', hidden=True, help='')
     loader.add_attr('jsonp_callback_default', 'callback', hidden=True, help='')
 
@@ -71,7 +73,7 @@ def wsd_config(loader):
     loader.add_attr('websocket_url_reconnect', '{ws_scheme}://{ws_host}{root_path_websocket}{session_id}/reconnect', help='')
     loader.add_attr('api_host', '{api_host}', help='')
     loader.add_attr('api_url_root', '{api_scheme}://{api_host}{root_path_api}', help='')
-    loader.add_attr('api_url_event', '{api_scheme}://{api_host}{root_path_api}{api_path_event}', help='')
+    #loader.add_attr('api_url_event', '{api_scheme}://{api_host}{root_path_api}{api_path_event}', help='')
 
 @config_callback('web_socket_server')
 def wsserver_config(loader):
@@ -94,7 +96,6 @@ class HttpdServer(object):
             self.favicon = f.read()
         self.wss = WebSocketServer(self.context)
         self.app = web.Application(loop=self.context.loop)
-        self.md5 = hashlib.md5()
 
     async def setup(self):
         await self.context.setup()
@@ -115,30 +116,53 @@ class HttpdServer(object):
     def setup_route(self):
         self.app.router.add_static(self.conf.root_path_static, path=str(self.context.resolve_local_path(self.conf.local_path_static)), name='static', show_index=True)
         self.app.router.add_get(self.conf.root_path_favicon, lambda request: web.Response(body=self.favicon, content_type='image/x-icon'))
+        
         for plugin, level in self.context.plugin_paths(self.conf.module_path_libs, self.conf.local_path_libs):
             for lib_file in [p for p in plugin.glob('**/*') if re.match('.*({})'.format('|'.join(self.conf.web_lib_file_extensions)),str(p))]:
                 with open(lib_file, 'rb') as f:
                     buf = f.read()
-                    self.md5.update(buf)
-                    checksum = self.md5.hexdigest()
+                    checksum = hashlib.md5(buf).hexdigest()
                     f = functools.partial((lambda src, mt, request : web.Response(body=src, content_type=mt, headers={'ETag':checksum})), buf, mimetypes.guess_type(str(lib_file))[0])
                 self.app.router.add_get(self._root_path_for_libs() + lib_file.name, f)
+        
         self.app.router.add_get(self._root_path_for_libs() + self.conf.relative_path_event_spec
                                 , lambda request: web.json_response([s._asdict() for s in self.context.event_handler_manager.specs]))
         self.app.router.add_get(self._root_path_for(self.conf.relative_path_web_service_discovery), self.get_web_service_discovery)
         self.app.router.add_get(self._root_path_for(self.conf.relative_path_web_service_main_jsonp), self.get_web_service_main_jsonp)
         self.app.router.add_get(self._root_path_for_websocket()+'{session_id}', self.wss.websocket_handler)
         self.app.router.add_get(self._root_path_for_websocket()+'{session_id}/reconnect', self.wss.websocket_handler_reconnect)
-        self.app.router.add_get(self._root_path_for_api()+self.conf.api_path_event, self.handle_event)
-        self.app.router.add_get(self._root_path_for_api()+self.conf.api_path_event_id, self.handle_event_id)
+        #self.app.router.add_get(self._root_path_for_api()+self.conf.api_path_event, self.handle_event)
+        #self.app.router.add_get(self._root_path_for_api()+self.conf.api_path_event_id, self.handle_event_id)
 
+        #router_methods = {kv[0]:kv[1] for kv in inspect.getmembers(self.app.router, inspect.ismethod)}
+        ducts_app = web.Application()
+        for route_add_type, path, methods, handler in self.context.event_handler_manager.web_handlers():
+            if route_add_type == 'add_route':
+                methods = '*' if methods == '' else methods
+                methods = [methods] if type(methods) is not list else methods
+                self.logger.debug('WEBAPI|PATH=%s|METHOD=%s', self._root_path_for_api()+path, methods)
+                self.logger.debug('WEBAPI|PATH=%s|METHOD=%s', path, methods)
+                for method in methods:
+                    #self.app.router.add_route(method, self._root_path_for_api()+path, handler)
+                    ducts_app.router.add_route(method, path, handler)
+        self.app.add_subapp(self._root_path_for_api(), ducts_app)
+                    
+        subapp = web.Application()
+        subapp.router.add_route('*', '/{path:.*}', self.hoge)
+        self.app.add_subapp('/hoge', subapp)
+
+    async def hoge(self, request):
+        self.logger.info('***HOGE***{}'.format(request))
+        ret = {k: "{}".format(getattr(request, k)) for k in dir(request)}
+        return web.json_response(ret)
+        
     async def run(self):
         if self.conf.behind_nginx:
             await aiohttp_remotes.setup(self.app, aiohttp_remotes.XForwardedRelaxed())
         self.handler = self.app.make_handler()
         self.srv = await self.context.loop.create_server(self.handler, self.conf.httpd_addr, self.conf.httpd_port)
         self.logger.notice('START|SOCKET=%s', self.srv.sockets[0].getsockname())
-        await self.context.redis.connect()
+        await self.context.run()
 
     async def close(self):
         self.logger.debug('CLOSEING|REDIS')
@@ -166,22 +190,22 @@ class HttpdServer(object):
         res.last_modified = datetime.now()
         return res
         
-    async def handle_event(self, request):
-        return web.json_response(self.context.event_handler_manager.key_ids)
+    # async def handle_event(self, request):
+    #     return web.json_response(self.context.event_handler_manager.key_ids)
 
-    async def handle_event_id(self, request):
-        operation = request.match_info['event_id']
-        params = request.rel_url.query['args'].strip().split()
-        return web.json_response({'result' : await self.redis.execute(operation, params)})
+    # async def handle_event_id(self, request):
+    #     operation = request.match_info['event_id']
+    #     params = request.rel_url.query['args'].strip().split()
+    #     return web.json_response({'result' : await self.redis.execute(operation, params)})
 
     async def _create_wsd_dict(self, request):
         params = self.conf._asdict()
         params['root_path_websocket'] = self._root_path_for_websocket()
-        params['ws_host'] = request.host
-        params['ws_scheme'] = 'wss' if request.secure else 'ws'
+        params['ws_host'] = self.conf.wsd_host if self.conf.wsd_host else request.host
+        params['ws_scheme'] = 'wss' if request.secure or self.conf.wsd_secure else 'ws'
         params['root_path_api'] = self._root_path_for_api()
-        params['api_host'] = request.host
-        params['api_scheme'] = request.scheme
+        params['api_host'] = self.conf.wsd_host if self.conf.wsd_host else request.host
+        params['api_scheme'] = 'https' if request.secure or self.conf.wsd_secure else 'http'
         params['session_id'] = await self.wss.init_session(request)
         your_wsd = namedtuple('wsd', ' '.join(self.wsd_dict.keys()))(*[v.format(**params) if type(v) is str else v for v in self.wsd_dict.values()])
         your_wsd_dict = your_wsd._asdict()
@@ -194,6 +218,7 @@ class WebSocketSession():
         self.context = context
         self.sid = sid
         self.context.session_store[sid] = self
+        self.send_lock = asyncio.Condition()
         self.reconnect_timeout = reconnect_timeout
         self.reconnect_lock = None
         self.reconnect_until = 0
@@ -231,6 +256,7 @@ class WebSocketSession():
         self.socket = web.WebSocketResponse()
         await self.socket.prepare(self.request)
         asyncio.ensure_future(self.ping())
+        asyncio.ensure_future(asyncio.gather(*[f for f in [h[1].handle_connected(self) for h in self.context.event_handler_manager.handlers()] if inspect.isawaitable(f)]))
         
         if reconnect:
             if self.reconnect_lock:
@@ -251,7 +277,7 @@ class WebSocketSession():
         self.context.session_store.pop(self.sid)
                 
     async def closed_by_client(self):
-        logger.debug("CLOSED_BY_CLIENT|SOCKET=%s|TRANSPORT=%s|IS_CLOGING=%s", self.socket.closed, self.request.transport, is_closing)
+        logger.debug("CLOSED_BY_CLIENT|SOCKET=%s|TRANSPORT=%s|IS_CLOING=%s", self.socket.closed, self.request.transport, is_closing)
 
     async def start(self):
         self.logger.debug('START')
@@ -292,12 +318,13 @@ class WebSocketSession():
                 if not self.socket.closed and self.request.transport is not None:
                     is_closing = self.request.transport.is_closing()
                     start = time.time()
-                    logger.debug("SEND_BYTES|TRANSPORT=%s|IS_CLOGING=%s", self.request.transport, is_closing)
+                    logger.debug("SEND_BYTES|TRANSPORT=%s|IS_CLOSING=%s", self.request.transport, is_closing)
                     try:
                         buf = msgpack.packb([request_id, event_id, data])
                         size = len(buf)
                         log.debug("SEND_BYTES|START|TIME=%s|DATA=%s|MSGPACK_SIZE=%s", time.time()-start, type(data), size)
-                        await asyncio.shield(self.socket.send_bytes(buf))
+                        async with self.send_lock:
+                            await asyncio.shield(self.socket.send_bytes(buf))
                         log.debug("SEND_BYTES|SENTB|TIME=%s", time.time()-start)
                         await asyncio.shield(self.socket.drain())
                         log.debug("SEND_BYTES|DRAIN|TIME=%s, NEXT=%s", time.time()-start, loop)
@@ -311,9 +338,11 @@ class WebSocketSession():
                         except Exception as e:
                             self.logger.exception("SOCKET_CLOSE_ERROR=%s", e)
                 logger.debug("SOCKET_CLOSED|TRANSPORT=%s", self.request.transport)
-                self.reconnect_lock = self.reconnect_lock if self.reconnect_lock else asyncio.Condition()
-                self.reconnect_until = self.reconnect_until if self.reconnect_until > 0 else datetime.now().timestamp() + self.reconnect_timeout
-                self.socket = None
+                if self.reconnect_lock is None:
+                    self.reconnect_lock = asyncio.Condition()
+                    self.reconnect_until = datetime.now().timestamp() + self.reconnect_timeout
+                    self.socket = None
+                    asyncio.ensure_future(asyncio.gather(*[f for f in [h[1].handle_wait_reconnect(self) for h in self.context.event_handler_manager.handlers()] if inspect.isawaitable(f)]))
             if self.reconnect_lock is not None:
                 lock = self.reconnect_lock
                 timeout = self.reconnect_until - datetime.now().timestamp()
@@ -511,16 +540,20 @@ class WebSocketServer(object):
             session.logger.exception("END|ERROR=%s", e)
             if not await session.is_closed():
                 event_id = -1 * event_id
-                ret = '{}:{}'.format(e.__class__.__name__, e.args)
-                await session.send_bytes_if_available(request_id, event_id, ret)
+                await session.send_bytes_if_available(request_id, event_id, e.__repr__())
 
 
     async def _event_handler_alive_monitoring(self, session, msg, request_id, event_id, data, handle, event):
         await session.start_alive_monitoring()
         closed = None
         try:
-            async for ret in handle(event):
+            gen = handle(event)
+            async for ret in gen:
                 if not await session.send_bytes_or_wait_reconnect(request_id, event_id, ret, True):
+                    try:
+                        gen.athrow(GeneratorExit)
+                    except GeneratorExit:
+                        pass
                     break
             session.logger.debug("START_CLOSE_HANDLERS")
             closed = await asyncio.gather(*[f for f in [h[1].handle_closed(session) for h in session.context.event_handler_manager.handlers()] if inspect.isawaitable(f)])
@@ -529,15 +562,25 @@ class WebSocketServer(object):
     
     async def _event_handler_async_for(self, session, msg, request_id, event_id, data, handle, event):
         session.logger.debug("START|AWAIT_ASYNC_LOOP")
-        async for ret in handle(event):
+        gen = handle(event)
+        async for ret in gen:
             if not await session.send_bytes_or_wait_reconnect(request_id, event_id, ret, True):
+                try:
+                    gen.athrow(GeneratorExit)
+                except GeneratorExit:
+                    pass
                 return False
         return True
 
     async def _event_handler_for(self, session, msg, request_id, event_id, data, handle, event):
         session.logger.debug("START|SYNC_LOOP")
-        for ret in handle(event):
+        gen = handle(event)
+        for ret in gen:
             if not await session.send_bytes_or_wait_reconnect(request_id, event_id, ret, True):
+                try:
+                    gen.throw(GeneratorExit)
+                except GeneratorExit:
+                    pass
                 return False
         return True
 
